import sys
import numpy as np

from hylaa.hybrid_automaton import HybridAutomaton
from hylaa.settings import HylaaSettings, PlotSettings, LabelSettings
from hylaa.core import Core
from hylaa.stateset import StateSet
from hylaa import lputil, aggstrat

def define_ha():
    '''make the hybrid automaton and return it'''

    ha = HybridAutomaton()

    # dynamics variable order: [e1,v1,a1,e2,v2,a2,e3,v3,a3, t, break_pattern_t, affine]
    
    communication = ha.new_mode('communication')
    
    a_matrix = [ \
        [0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], \
        [0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0], \
        [1.605, 4.868, -3.5754, -0.8198, 0.427, -0.045, -0.1942, 0.3626, -0.0946, 0, 0, 0], \
        [0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0], \
        [0, 0, 1, 0, 0, -1, 0, 0, 0, 0, 0, 0], \
        [0.8718, 3.814, -0.0754, 1.1936, 3.6258, -3.2396, -0.595, 0.1294, -0.0796, 0, 0, 0], \
        [0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0], \
        [0, 0, 0, 0, 0, 1, 0, 0, -1, 0, 0, 0], \
        [0.7132, 3.573, -0.0964, 0.8472, 3.2568, -0.0876, 1.2726, 3.072, -3.1356, 0, 0, 0], \
        [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1], \
        [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1], \
        [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], \
        ]
    communication.set_dynamics(a_matrix)
    
    #input u in [-9, 1] for v1'
    b_mat = [[0], [1], [0], [0], [0], [0], [0], [0], [0], [0], [0], [0]]
    b_constraints = [[1], [-1]]
    b_rhs = [1, 9]
    communication.set_inputs(b_mat, b_constraints, b_rhs)

    # t <= 20, break_pattern_t <= 5
    communication.set_invariant([[0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0]], [20])
    communication.set_invariant([[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0]], [5])

    no_communication = ha.new_mode('no_communication')
    a_matrix = [ \
        [0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], \
        [0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0], \
        [1.605, 4.868, -3.5754, 0, 0, 0, 0, 0, 0, 0, 0, 0], \
        [0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0], \
        [0, 0, 1, 0, 0, -1, 0, 0, 0, 0, 0, 0], \
        [0, 0, 0, 1.1936, 3.6258, -3.2396, 0, 0, 0, 0, 0, 0], \
        [0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0], \
        [0, 0, 0, 0, 0, 1, 0, 0, -1, 0, 0, 0], \
        [0.7132, 3.573, -0.0964, 0.8472, 3.2568, -0.0876, 1.2726, 3.072, -3.1356, 0, 0, 0], \
        [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1], \
        [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1], \
        [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], \
        ]

    no_communication.set_dynamics(a_matrix)
    # input u in [-9, 1] for v1'
    b_mat = [[0], [1], [0], [0], [0], [0], [0], [0], [0], [0], [0], [0]]
    b_constraints = [[1], [-1]]
    b_rhs = [1, 9]
    no_communication.set_inputs(b_mat, b_constraints, b_rhs)

    # t <= 20, break_pattern_t <= 5
    communication.set_invariant([[0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0]], [20])
    communication.set_invariant([[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0]], [5])

    _error = ha.new_mode('_error')


    # transitions

    trans = ha.new_transition(communication, no_communication)
    # break_pattern_t >= 5.0
    trans.set_guard([[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0], ], [-5, ])
    # reset: break_pattern_t = 0
    reset_csr = [[1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], \
                 [0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], \
                 [0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0], \
                 [0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0], \
                 [0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0], \
                 [0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0], \
                 [0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0], \
                 [0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0], \
                 [0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0], \
                 [0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0], \
                 [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], \
                 [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1], \
                 ]
    trans.set_reset(reset_csr)

    trans = ha.new_transition(no_communication, communication)
    # break_pattern_t >= 5.0
    trans.set_guard([[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0], ], [-5, ])
    # reset: break_pattern_t = 0
    reset_csr = [[1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], \
                 [0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], \
                 [0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0], \
                 [0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0], \
                 [0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0], \
                 [0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0], \
                 [0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0], \
                 [0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0], \
                 [0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0], \
                 [0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0], \
                 [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], \
                 [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1], \
                 ]
    trans.set_reset(reset_csr)

    ha.new_transition(communication, _error).set_guard([[1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], ], [-30, ])
    ha.new_transition(communication, _error).set_guard([[0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0], ], [-30, ])
    ha.new_transition(communication, _error).set_guard([[0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0], ], [-30, ])

    ha.new_transition(no_communication, _error).set_guard([[1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], ], [-30, ])
    ha.new_transition(no_communication, _error).set_guard([[0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0], ], [-30, ])
    ha.new_transition(no_communication, _error).set_guard([[0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0], ], [-30, ])   

    return ha

def define_init_states(ha):
    '''returns a list of StateSet objects'''
    
    #variable order: [e1,v1,a1,e2,v2,a2,e3,v3,a3, t, break_pattern_t, affine]    rv = []
#    rv = []
    # all 0, except affine = 1.0
    mode = ha.modes['communication']
#    mat = [[1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], \
#        [-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], \
#        [0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], \
#        [0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], \
#        [0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0], \
#        [0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0], \
#        [0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0], \
#        [0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0], \
#        [0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0], \
#        [0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0], \
#        [0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0], \
#        [0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0], \
#        [0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0], \
#        [0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0], \
#        [0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0], \
#        [0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0], \
#        [0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0], \
#        [0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0], \
#        [0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0], \
#        [0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0], \
#        [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0], \
#        [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0], \
#        [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1], \
#        [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1], ] \

#   rhs = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,-1, ]
#    rv.append(StateSet(lputil.from_constraints(mat, rhs, mode), mode))
    
#    return rv
    init_lpi = lputil.from_box([(0, 0), (0, 0), (0, 0), (0, 0), (0, 0), (0, 0), (0, 0), (0, 0), (0, 0), (0, 0), (0, 0), (1, 1)], mode)
    
    init_list = [StateSet(init_lpi, mode)]

    return init_list

def define_settings(image_path):
    '''get the hylaa settings object
    see hylaa/settings.py for a complete list of reachability settings'''

    # step_size = 0.05, max_time = 20.0
    settings = HylaaSettings(0.1, 20.0)
    settings.plot.plot_mode = PlotSettings.PLOT_NONE
    
    if image_path is not None:
        settings.plot.filename = image_path
        settings.plot.plot_mode = PlotSettings.PLOT_IMAGE
        settings.plot.xdim_dir = 9
        settings.plot.ydim_dir = 0
        settings.plot.label = LabelSettings()
        settings.plot.label.x_label = 't'
        settings.plot.label.y_label = 'e1'

    return settings

def run_hylaa(image_path):
    'runs hylaa, returning a HylaaResult object'
    ha = define_ha()
    init = define_init_states(ha)
    settings = define_settings(image_path)

    result = Core(ha, settings).run(init)

    return result

if __name__ == '__main__':
    image_path = None
    
    if len(sys.argv) > 1 and sys.argv[1].endswith('.png'):
        image_path = sys.argv[1]
    
    run_hylaa(image_path)
